package cz.cca.training.rest;

import cz.cca.training.dao.EmployeeDAO;
import cz.cca.training.entity.EmployeeEO;
import cz.cca.training.rest.response.EmployeeJSON;
import cz.cca.training.rest.response.EmployeeNewJSON;
import cz.cca.training.rest.response.EmployeeUpdateJSON;
import cz.cca.training.rest.response.EmployeesBrowseJSON;
import cz.cca.training.rest.response.LeaderJSON;
import cz.cca.training.utils.MapFromJson;
import cz.cca.training.utils.MapToJsonObject;
import java.util.LinkedList;
import java.util.List;
import javax.inject.Inject;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Response;

/**
 * <p>
 * Title: {@link EmployeeREST}
 * </p>
 * <p>
 * Description: Rest service for basic work about employees with necessary methods.
 * </p>
 * <p>
 * Copyright: Copyright (c) 2015 Kosek David
 * </p>
 * <p>
 * ZCU KIV/FAV
 * </p>
 *
 * @author david.kosek A14N0132P
 * @date 25. 3. 2015 11:48:53
 */
@Path("employee")
public class EmployeeREST {

    @Inject
    private EmployeeDAO dao;

    /**
     * Get all employee (example URI: http://localhost:8080/training/api/employee)
     * @return list of employee in JSON
     */
    @GET 
    @Produces("application/json")
    public List<EmployeeJSON> getEmployees() {

        List<EmployeeEO> employeeEOList = dao.getAllEmployees();
        List<EmployeeJSON> employeeJSONList = new LinkedList<EmployeeJSON>();

        for (EmployeeEO employeeEO : employeeEOList) {
           employeeJSONList.add(MapToJsonObject.mapToEmployeeJSON(employeeEO));
        }

        return employeeJSONList;
    }

    /**
     * Get one employee (detail) according the identifier (example URI: http://localhost:8080/training/api/employee/detail/3)
     * @param id the identifier
     * @return employee in JSON
     */
    @GET
    @Path("/detail/{id}") 
    @Produces("application/json")
    public EmployeeJSON getEmployee(@PathParam("id") int id) {
        return MapToJsonObject.mapToEmployeeJSON(dao.getEmployee(id));
    }    
    
    /**
     * Get all employee for browse - not all property (example URI: http://localhost:8080/training/api/employee/browseEmployees/)
     * @return list of employee in JSON
     */
    @GET
    @Path("/browseEmployees")
    @Produces("application/json")
    public List<EmployeesBrowseJSON> getEmployeesBrowse() {

        List<EmployeeEO> listEO = dao.getAllEmployeesForBrowse();
        List<EmployeesBrowseJSON> listJSON = new LinkedList<EmployeesBrowseJSON>();

        for (EmployeeEO item : listEO) {
            listJSON.add(MapToJsonObject.mapToEmployeesBrowseJSON(item, this.dao.getEmployee(item.getIdLeader())));
        }

        return listJSON;
    }
    
    /**
     * Delete employee according id (example URI: http://localhost:8080/training/api/employee/delete/3).
     * @param idEmployee the identifier
     */
    @DELETE
    @Path("/delete/{id}")
    public void deleteEmployee(@PathParam("id") int idEmployee) {
       this.dao.deleteEmployee(this.dao.getEmployee(idEmployee));
    }
    
    /**
     * Insert new employee (example URI: http://localhost:8080/training/api/employee/create/).
     * @param emp new user from PL
     * @return response (expected all is alright, that's why sending ok)
     */
    @PUT
    @Consumes("application/json")
    @Path("/create")
    public Response createEmployee(EmployeeNewJSON emp) {
        EmployeeEO newEmp = MapFromJson.mapFromEmployeeNewJSON(emp, this.dao.getCenter(emp.getIdCenter()), this.dao.getTypeRelationship(emp.getIdTypeRelationship()));
        this.dao.saveEmployee(newEmp);
        return Response.ok().build();   
    }
    
    
    /**
     * Update old employee (example URI: http://localhost:8080/training/api/employee/).
     * @param emp new user's values from PL
     * @return response (expected all is alright, that's why sending ok)
     */
    @POST
    @Consumes("application/json") 
    public Response updateEmployee(EmployeeUpdateJSON emp) {
        EmployeeEO newEmp = MapFromJson.mapFromEmployeeUpdateJSON(emp, this.dao.getEmployee(emp.getIdEmployee()), this.dao.getCenter(emp.getIdCenter()), this.dao.getTypeRelationship(emp.getIdTypeRelationship()));
        this.dao.updateEmployee(newEmp);
        return Response.ok().build();
    }
    
    /**
     * Get all info about leaders (example URI: http://localhost:8080/training/api/employee/leaders/).
     * @return list of leaders in JSON
     */
    @GET 
    @Path("/leaders")
    @Produces("application/json")
    public List<LeaderJSON> getLeaders() {

        List<EmployeeEO> employeeEOList = this.dao.getAllEmployees();
        List<LeaderJSON> leaderJSONList = new LinkedList<LeaderJSON>();

        for (EmployeeEO employeeEO : employeeEOList) {
           leaderJSONList.add(MapToJsonObject.mapToLeaderJSON(employeeEO));
        }

        return leaderJSONList;
    }
    
    
    

}
